import requests
from requests.adapters import HTTPAdapter
from seetaas_helper.config import get_metric_api, get_metric_token
import logging
import sys

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
logger = logging.getLogger("seetaas-helper")

session = requests.Session()
session.mount('http://', HTTPAdapter(max_retries=3))
session.mount('https://', HTTPAdapter(max_retries=3))


class Config:
    OPEN = True


def TURN_OFF():
    Config.OPEN = False


class _MsgType:
    NewMetric = "NewMetric"
    MetricData = "MetricData"


class _ResultType:
    SCALAR_RESULT = 'scalar_result'  # 用于如Rank1，Rank5，LFW，MegaFace测试协议输出的单精度值测试结果
    CURVE_RESULT = 'curve_result'  # 用于如ROC测试协议输出的测试曲线
    PROGRESS = 'progress'


_metric_api = get_metric_api()
_metric_token = get_metric_token()


def _send_metric(body):
    if not Config.OPEN:
        return
    if len(_metric_api) == 0 or len(_metric_token) == 0:
        raise Exception("You should run your algorithm inner SeeTaaS or AutoDL platform")
    try:
        if not isinstance(body, list):
            body = [body]
        resp = session.post('{}/uploadTaskMetrics'.format(_metric_api),
                            json={
                                "token": _metric_token,
                                "items": body
                            },
                            timeout=5)
        if resp.status_code != 200:
            logger.error("send metrics http code: {}. content: {}".format(resp.status_code, resp.content))
    except requests.RequestException as e:
        logger.error('Could not reach metric api. detail: {}'.format(e))


def _send_result(result_type, name, value):
    if not Config.OPEN:
        return
    if len(_metric_api) == 0 or len(_metric_token) == 0:
        raise Exception("You should run your algorithm inner SeeTaaS or AutoDL platform")
    try:
        resp = session.post('{}/updateTaskAttribute'.format(_metric_api),
                            json={
                                "token": _metric_token,
                                "type": result_type,
                                "name": name,
                                "value": value
                            },
                            timeout=5)
        if resp.status_code != 200:
            logger.error("send evaluate result http code: {}. content: {}".format(resp.status_code, resp.content))
    except requests.RequestException as e:
        logger.error('Could not reach evaluate result api. detail: {}'.format(e))



from __future__ import unicode_literals
import os
import multiprocessing
import gunicorn.app.base
from gunicorn.six import iteritems
from seetaas_helper.base import logger
from flask import Flask
from flask import request, jsonify, abort
from flask_cors import CORS
app = Flask('seetaas-deploy')
CORS(app, supports_credentials=True)


class _StandaloneApplication(gunicorn.app.base.BaseApplication):

    def __init__(self, app, options=None):
        self.options = options or {}
        self.application = app
        super(_StandaloneApplication, self).__init__()

    def init(self, parser, opts, args):
        pass

    def load_config(self):
        config = dict([(key, value) for key, value in iteritems(self.options)
                       if key in self.cfg.settings and value is not None])
        for key, value in iteritems(config):
            self.cfg.set(key.lower(), value)

    def load(self):
        return self.application


@app.route("/", methods=['POST'])
def _handle_image():
    logger.info("receive request")
    try:
        req = request.get_json(force=True)
        base64_image = req['base64_image']
    except KeyError:
        logger.error("not found base64 image")
        return abort(400)
    if len(base64_image) == 0:
        logger.error("base64 image is null")
        return abort(400)
    logger.info("algorithm infer...")
    resp = _Deploy.func(base64_image)
    logger.info("infer response: {}".format(resp))
    return jsonify(resp)


class _Deploy:
    func = None

    @staticmethod
    def register(handler):
        if not callable(handler):
            raise Exception("Handler must be callable")
        _Deploy.func = handler

    @classmethod
    def run(cls, num_worker=1):
        options = {
            'bind': '%s:%s' % ('0.0.0.0', '5050'),
            'workers': num_worker or cls.num_of_workers(),
        }
        if not callable(_Deploy.func):
            raise Exception("Please register your handle function begin run")
        _StandaloneApplication(app, options).run()

    @classmethod
    def num_of_workers(cls):
        workers = os.environ["AUTOCNN_WORKERS"] or 1
        if isinstance(workers, str) and workers.isdigit():
            workers = int(workers)
        assert workers > 0, ValueError
        return workers if workers < multiprocessing.cpu_count() * 2 else multiprocessing.cpu_count() * 2


register_handle = _Deploy.register
run_server = _Deploy.run


if __name__ == '__main__':
    @register_handle
    def handle(base64_str):
        return {
            "object_list": [
                {
                    "class_id": 1,
                    "score": 0.98,
                    "location": [33, 23, 45, 74]
                }
            ]
        }
    _Deploy.run()



from seetaas_helper.config import (
    get_parameter,
    write_parameter,
    get_data_attribute,
    get_output_dir,
    get_input_dir,
)

from seetaas_helper import config, deploy, metric, result, progress, dataset
from seetaas_helper.base import TURN_OFF


#!/usr/bin/env python

from setuptools import setup, find_packages


setup(name='seeta-helper',
      version='0.0.2',
      platforms='any',
      maintainer_email='seetaas@seetatech.com',
      description='SeeTaaS helper functions',
      packages=find_packages(),
      install_requires=[
          'requests==2.18.4',
          'flask==0.12.2',
          'flask-cors==3.0.8',
          'gunicorn==19.7.1',
      ],
      entry_points={
          "console_scripts": [
              "helper = seetaas_helper.config:write_parameter",
          ],
      },
      classifiers=[
          'Programming Language :: Python',
          'Operating System :: OS Independent',
      ])



from seetaas_helper.base import session, Config
import requests
import json
from seetaas_helper.config import get_dataset_api, get_dataset_token
import logging
logger = logging.getLogger("seetaas-helper")


def send_data_attribute(output_index=1, class_num=None, class_name_dict=None, sample_num=None):
    """
    将数据的属性信息发送给服务器
    :param output_index:    第index个输出，如划分训练集和测试集会有多个输出. 从1开始
    :param class_num:   类别数量.
    :param class_name_dict:  id与类名对应字典. 如: {0: "airplane", 1: "sofa", ...}
    :param sample_num:  数据集包含样本总数. 注意: 样本总数并非文件总数，如检测数据集，样本总数=图片文件总数=XML标注文件总数
    :return:
    """
    if not Config.OPEN:
        return
    dataset_api = get_dataset_api()
    for k, v in {"class_name": class_num, "class_name_dict": class_name_dict, "sample_num": sample_num}.items():
        try:
            resp = session.post('{}/updateDatasetAttribute'.format(dataset_api),
                                json={
                                    "token": get_dataset_token(),
                                    "output": str(output_index),
                                    "name": k,
                                    "value": json.dumps(v)
                                },
                                timeout=5)
            logger.info("send data attribute, {} = {}".format(k, v))
            if resp.status_code != 200:
                logger.error("send data attribute http code: {}. content: {}".format(resp.status_code, resp.content))
        except requests.RequestException as e:
            logger.error('Could not reach dataset api. detail: {}'.format(e))
